@@ -8,7 +8,6 @@
 
 OIDN_NAMESPACE_BEGIN
 
-  Ref<Conv> newHIPConvDL(HIPEngine* engine, const ConvDesc& desc);
-  Ref<Conv> newHIPConvWMMA(HIPEngine* engine, const ConvDesc& desc);
+  Ref<Conv> newHIPConv(HIPEngine* engine, const ConvDesc& desc);
 
 OIDN_NAMESPACE_END
