@@ -1,4 +1,4 @@
-## Copyright 2018-2021 Intel Corporation
+## Copyright 2018 Intel Corporation
 ## SPDX-License-Identifier: Apache-2.0
 
 import os
@@ -33,6 +33,8 @@ def get_channels(features, target):
     channels += ['alb.r', 'alb.g', 'alb.b']
   if 'nrm' in features:
     channels += ['nrm.x', 'nrm.y', 'nrm.z']
+  if 'z' in features:
+    channels += ['z']
   return channels
 
 def get_dataset_channels(features):
@@ -70,7 +72,7 @@ def get_image_feature(filename):
     return 'srgb' # no extension, assume sRGB
   else:
     ext = filename_split[-1].lower()
-    if ext in {'exr', 'pfm', 'hdr'}:
+    if ext in {'exr', 'pfm', 'phm', 'hdr'}:
       if len(filename_split) == 3:
         feature = filename_split[-2]
         if feature in {'sh1x', 'sh1y', 'sh1z'}:
@@ -81,27 +83,29 @@ def get_image_feature(filename):
     else:
       return 'srgb' # assume sRGB
 
-# Loads image features in EXR format with given filename prefix
+# Loads image features in EXR format with given filename prefix, and returns the concatenated image
+# and the number of channels loaded for each feature
 def load_image_features(name, features):
   images = []
+  num_channels = {}
 
   # HDR color
   if 'hdr' in features:
-    hdr = load_image(name + '.hdr.exr', num_channels=3)
+    hdr, num_channels['hdr'] = load_image(name + '.hdr.exr', num_channels=3)
     hdr = np.maximum(hdr, 0.)
     images.append(hdr)
 
   # LDR color
   if 'ldr' in features:
-    ldr = load_image(name + '.ldr.exr', num_channels=3)
+    ldr, num_channels['ldr'] = load_image(name + '.ldr.exr', num_channels=3)
     ldr = np.clip(ldr, 0., 1.)
     images.append(ldr)
 
   # SH L1 color coefficients
   if 'sh1' in features:
-    sh1x = load_image(name + '.sh1x.exr', num_channels=3)
-    sh1y = load_image(name + '.sh1y.exr', num_channels=3)
-    sh1z = load_image(name + '.sh1z.exr', num_channels=3)
+    sh1x, num_channels['sh1x'] = load_image(name + '.sh1x.exr', num_channels=3)
+    sh1y, num_channels['sh1y'] = load_image(name + '.sh1y.exr', num_channels=3)
+    sh1z, num_channels['sh1z'] = load_image(name + '.sh1z.exr', num_channels=3)
 
     for sh1 in [sh1x, sh1y, sh1z]:
       # Clip to [-1..1] range (coefficients are assumed to be normalized)
@@ -114,22 +118,36 @@ def load_image_features(name, features):
 
   # Albedo
   if 'alb' in features:
-    albedo = load_image(name + '.alb.exr', num_channels=3)
+    albedo, num_channels['alb'] = load_image(name + '.alb.exr', num_channels=3)
     albedo = np.clip(albedo, 0., 1.)
     images.append(albedo)
 
   # Normal
   if 'nrm' in features:
-    normal = load_image(name + '.nrm.exr', num_channels=3)
+    normal, num_channels['nrm'] = load_image(name + '.nrm.exr', num_channels=3)
     normal = np.clip(normal, -1., 1.)
-    
+
     # Transform to [0..1] range
     normal = normal * 0.5 + 0.5
 
     images.append(normal)
 
+  # Depth
+  if 'z' in features:
+    depth, num_channels['z'] = load_image(name + '.z.exr', num_channels=1)
+    depth = np.maximum(depth, 0.)
+
+    # If this is an auxiliary feature, transform to [0..1] range
+    # Otherwise, the transfer function will be applied later
+    if get_main_feature(features) != 'z':
+      depth = torch.from_numpy(depth)
+      depth = LogTransferFunction().forward(depth)
+      depth = depth.numpy()
+
+    images.append(depth)
+
   # Concatenate all feature images into one image
-  return np.concatenate(images, axis=2)
+  return np.concatenate(images, axis=2), num_channels
 
 # Tries to load metadata for an image with given filename/prefix, returns None if it fails
 def load_image_metadata(name):
@@ -223,7 +241,9 @@ def get_data_loader(rank, cfg, dataset, shuffle=False):
                       sampler=sampler,
                       shuffle=(shuffle if sampler is None else False),
                       num_workers=cfg.num_loaders,
-                      pin_memory=(cfg.device != 'cpu'))
+                      persistent_workers=True,
+                      pin_memory=(cfg.device != 'cpu'),
+                      drop_last=True)
 
   return loader, sampler
 
@@ -248,18 +268,22 @@ def get_preproc_data_dir(cfg, name):
       # Backward compatibility
       if not hasattr(data_cfg, 'clean_aux'):
         data_cfg.clean_aux = False
+      if not hasattr(data_cfg, 'aux_results'):
+        data_cfg.aux_results = []
 
       # Check whether the dataset matches the requirements
       if get_main_feature(data_cfg.features) == get_main_feature(cfg.features) and \
         all(f in data_cfg.features for f in cfg.features) and \
         data_cfg.clean_aux == cfg.clean_aux and \
+        (not data_cfg.aux_results if not cfg.aux_results else \
+         all(r in data_cfg.aux_results for r in cfg.aux_results)) and \
         data_cfg.transfer == cfg.transfer:
         # Select the most recent version with the minimal amount of channels stored
         num_channels = len(get_dataset_channels(data_cfg.features))
         if best_dir is None or num_channels <= best_num_channels:
           best_dir = data_dir
           best_num_channels = num_channels
-  
+
   if best_dir is None:
     error('no matching preproccessed dataset found')
   return best_dir
@@ -403,6 +427,9 @@ class TrainingDataset(PreprocessedDataset):
 
     # DEBUG: Save the tile
     #save_image('tile_%d.png' % index, target_image)
+    #save_image('tile_%d_col.png' % index, input_image[..., 0:3])
+    #save_image('tile_%d_alb.png' % index, input_image[..., 3:6])
+    #save_image('tile_%d_nrm.png' % index, input_image[..., 6:9])
 
     # Convert the tiles to tensors
     return image_to_tensor(input_image), image_to_tensor(target_image)
@@ -453,7 +480,7 @@ class ValidationDataset(PreprocessedDataset):
               self.tiles.append((sample_index, oy, ox, ch))
           else:
             self.tiles.append((sample_index, oy, ox, input_channel_indices))
-      
+
   def __len__(self):
     return len(self.tiles)
 
